/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#include <htc.h>           /* Global Header File */
#include <stdint.h>        /* For uint8_t definition */
#include <stdbool.h>       /* For true/false definition */

#include "system.h"        /* System funct/params, like osc/peripheral config */
#include "user.h"          /* User funct/params, such as InitApp */


/******************************************************************************/
/* Configuration Bits                             */
/******************************************************************************/

__CONFIG(FOSC_INTOSC & WDTE_OFF & PWRTE_OFF & MCLRE_OFF & BOREN_OFF);
__CONFIG(WRT_OFF & STVREN_OFF & LVP_OFF);

/******************************************************************************/
/* User Global Variable Declaration                                           */
/******************************************************************************/
#define BUF_SIZE  16
#define OUT_BUF_SIZE 256
#define AD_SIZE  32

/* i.e. uint8_t <variable_name>; */
unsigned int tick_count;
unsigned int sec_cnt;
unsigned int ad[AD_SIZE];

unsigned int buf_index;
unsigned int buf[BUF_SIZE]; //temp buffer
unsigned int outbuf[OUT_BUF_SIZE];  //ready to send words by address


int temp;
int add;
int chan;


/******************************************************************************/
/* Main Program                                                               */
/******************************************************************************/

uint8_t main(void)
{
    int i;
    int thischan;
    /* Configure the oscillator for the device */
    OSCCON = 0b01111000; //i2c needs 16 Meg

    /* Initialize I/O and Peripherals for application */
    InitApp();

    /* TODO <INSERT USER APPLICATION CODE HERE> */
    temp = 0;
    for (i=0; i<OUT_BUF_SIZE; ++i) //clear buffer
    {
        outbuf[i] = 1000 + i;
    }
    while(1)
    {
        tick_count = 0;
        ++sec_cnt;

        if (temp>0)
            --temp;

/*        if (((sec_cnt%2)==0) || (temp>0))
            PORTA = 0b00010000;
        else
            PORTA = 0b00000000;
*/
        //test code turn on led
        if (outbuf[0x34]>0)
            PORTA = 0b00010000;
        else
            PORTA = 0b00000000;

        //second counter
        while (tick_count < 30)  //wait here until a second is finished
        {
            //do A/D's
            ++chan;
            if (chan>1)
                chan = 0;
            //chan is terminal number 0 to 21, this chan is pic pin
            if (chan == 0)
                thischan = 5;
            else if (chan == 1)
                thischan = 3;


            ADCON0 = (thischan << 2) | 0b00000001;  //set channel
            for (i=0; i<100; ++i)  //delay
            {
                ++i;
            }

            ADCON0 = ADCON0 | 0b00000011;   //start a/d
            while (ADCON0 & 0b00000010) //wait for conversion
            {
            }
            outbuf[chan] = ADRESH<<8 | ADRESL;
        }
    }

}


void InitApp(void)
{
    /* TODO Initialize User Ports/Peripherals/Project here */


    /* Setup analog functionality and port direction */
//    TRISA4 = 0; //output led
    TRISC3 = 1; //i2c must be input
    TRISC4 = 1; //i2c must be input
    ANSC3 = 0;  //digital i2c
    ANSC4 = 0;  //digital i2c

    /* Initialize ic2 */
    SSPSTAT = 0b00000000;
    SSPCON1 = 0b00110110;
    SSPCON2 = 0b00000001;
    SSPCON3 = 0b00000011;
    SSPMSK  = 0b11111100;
    SSPADD  = 0x20;

    /* Enable timer 0 */
    OPTION_REG = 0b11010111;

    /* A to D */
    //io pins default to analog mode after reset
    ADCON0 = 0b00000001;
    ADCON1 = 0b10000000;

    /* Enable interrupts */
    PEIE = 1; //peripheral interupts enabled (includes i2c)
    SSPIE = 1;  //i2c interupt
    TMR0IE = 1; //enable timer 0 interupt
    GIE = 1;  //general enable interupts
}


void interrupt isr(void)
{
    unsigned int i, stat;
    unsigned int dat;
    /* This code stub shows general interrupt handling.  Note that these
    conditional statements are not handled within 3 seperate if blocks.
    Do not use a seperate if block for each interrupt flag to avoid run
    time errors. */


    //python write word
    //i2c.write_word_data (address, command, data)
    //sequesnce:
    //1: state 1, buf contains address
    //2: state 2, buf contains command
    //3: state 2, first byte
    //4: state 2, second byte
    //state 5 gets called at end of message

    //python to read word from pic
    //i2c.read_word_data (address, command)
    //sequence:
    //1: state 1, buf contains address
    //2: state 2, buf contains command
    //3: state 3, first byte
    //4: state 4, second byte
    //state 5 gets called at end of message

    if (SSPIF == 1) //i2c interupt
    {
        if (SSPOV == 1)
        {
            SSPOV = 0;  //clear overflow
            dat = SSPBUF;
            temp = 30;
        }
        else
        {
            stat = SSPSTAT;
            stat = stat & 0b00101101;
            //find which state we are in
            //state 1 Master just wrote our address
            if ((stat ^ 0b00001001) == 0) //S=1 && RW==0 && DA==0 && BF==1
            {
                for (i=0; i<BUF_SIZE; ++i) //clear buffer
                    buf[i] = 0;
                buf_index = 0;  //clear buffer index
                add = SSPBUF; //read address so we know later if read or write
            }
            //state 2 Master just wrote data
            else if ((stat ^ 0b00101001) == 0) //S=1 && RW==0 && DA==1 && BF==1
            {
                dat = SSPBUF;
                if (buf_index >= BUF_SIZE)
                    buf_index = 0;
                buf[buf_index] = dat;
                ++buf_index;
                if (buf_index==3) //recieved 2 bytes so store
                {
                    if (buf[0]<OUT_BUF_SIZE)
                    {
                        dat = buf[2] * 256 + buf[1];
                        outbuf[buf[0]] = dat;
                    }
                }
            }
            //state 3 Master want to read, just wrote address
            else if ((stat ^ 0b00001100) == 0) //S=1 && RW==1 && DA==0 && BF==0
            {
                add = SSPBUF; //keep the address
                buf_index = buf[0];  //index to read
                if (buf_index >= OUT_BUF_SIZE)
                    buf_index = 0;
                WCOL = 0;   //clear write collision flag
                dat = outbuf[buf_index]; //low byte
                SSPBUF = dat & 0x00FF; //data to send
            }
            //state 4 Master want to read, last byte was already data
            else if ((stat ^ 0b00101100) == 0) //S=1 && RW==1 && DA==1 && BF==0
            {
                if (buf_index >= OUT_BUF_SIZE)
                    buf_index = 0;
                WCOL = 0;   //clear write collision flag
                dat = outbuf[buf_index] >> 8;
                SSPBUF = dat & 0x00FF; //data to send, low byte
                ++buf_index;
            }
            //state 5 Master sends NACK to end message
            else if ((stat ^ 0b00101000) == 0) //S=1 && RW==0 && DA==1 && BF==0
            {
                dat = SSPBUF;
            }
            else //undefined, clear buffer
            {
                dat = SSPBUF;
            }
        }
    CKP = 1; //release the clk
    SSPIF = 0; //clear interupt flag
    }
    else if (TMR0IF == 1) //timer 0 interupt
    {
        TMR0IF = 0;
        ++tick_count;
    }



}
